/*
 * $Id: addvutil.C,v 1.14 1995/05/26 15:55:06 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "addvutil.H"

/***************************************************************************/
/*
 * Private utilities to help with the utilities
 */

int null_strlen( char *s ){
  /*
   * Check to make sure s is non-NULL before a length is returned.
   * If s is NULL, the return value is 0, otherwise the return value
   * is the same as strlen()
   */
  int ret;

  if( s ){
    ret = strlen(s);
  } else {
    ret = 0;
  }

  return ret;
}

/***************************************************************************/

char *null_strdup( char *s ){
  /*
   * Duplicate a string, allocating space with malloc().
   * if "s" is NULL, return NULL
   */
  char *ret;
  int size;

  if( s ){
    size = strlen( s ) + 1;    // DON'T USE null_strlen() here
    ret = (char *)malloc( size );
    if( ret ){
      strcpy( ret, s );
    }
  } else {
    ret = NULL;
  }

  return ret;
}

/***************************************************************************/

char last_char( char *s ){
  /*
   * Return the last character in the string.
   * Return '\0' if s == NULL
   */
  char c;
  int  l;

  l = null_strlen( s );
  if( l ){
    c = s[ l-1 ];
  } else {
    // Empty string or NULL
    c = '\0';
  }

  return c;
}

/***************************************************************************/

char *itoa( int num ){
  /*
   * Convert the number into a static string.  DO NOT FREE
   */

  static char numstr[10];

  sprintf( numstr, "%d", num );
  return numstr;
}

/***************************************************************************/

char *make_fname( char *path, char *file, char *ext ){
  /*
   * Return a newly allocated string containing the concatenation
   * of the fields correctly.  (Add slash and period if necessary.)
   * Do not try to add an extension if NULL.
   */
  /*
   * We should make sure the path has a
   * trailing slash (and put it in if not), and make sure the extension
   * has a period (and put it in the string if not).
   */
  char *ret;       // String to return
  int  len;        // Length to create
  char *newpath;   // A copy of the path parameter
  int  newpathlen;

  // Allocate space
  len = null_strlen(path) + null_strlen(file) + null_strlen(ext) + 5;
  ret = (char *)malloc( len );

  newpathlen = null_strlen(path) + 1;
  newpath = (char *)malloc( newpathlen );
  if( path ){
    strcpy( newpath, path );
  } else {
    strcpy( newpath, "" );
  }

  if( ret ){
    // Allocated ok
    char slash[2];   // Slash after path if needed
    char dot[2];     // Dot before extension if needed

    // Check to see what, if anything, needs to be put after the path
    if( !path ){
      // Use a relative path
      strcpy( slash, "" );
    } else if( (last_char( path ) == '/') || (file[0] == '/') ){
      // There is a slash there
      strcpy( slash, "" );
    } else {
      // We need to add a slash
      strcpy( slash, "/" );
    }

    if( ext ){
      // There is an extension
      // Check to see what, if anything, needs to be put before the extension
      if( ext[0] == '.' ){
        // There is a dot there
        strcpy( dot, "" );
      } else {
        // We need to add a dot
        strcpy( dot, "." );
      }
      
      // Create the string
      sprintf( ret, "%s%s%s%s%s", newpath, slash, file, dot, ext );
    } else {
      // No extension
      // Create the string
      sprintf( ret, "%s%s%s", newpath, slash, file );
    }      
  } else {
    // Out of memory?
  }

  free( newpath );
  
  return ret;
}

//---------------------------------------------------------------------------

char *make_fname( char *path, int room, char *ext ){
  /*
   * Return a newly allocated string containing the path and file of
   * the room with the required extension.  Return NULL if failed.
   */
  /*
   * We should convert the number to a string, make sure the path has a
   * trailing slash (and put it in if not), and make sure the extension
   * has a period (and put it in the string if not).  (Most of this is
   * done in the above routine.)
   */

  char *ret;       // String to return
  char *roomstr;   // String with the room number.  Do not free

  // Convert room number to string
  roomstr = itoa( room );

  ret = make_fname( path, roomstr, ext );

  return ret;
}

/***************************************************************************/

char *get_path( char *pathfile ){
  /*
   * Return a newly allocated string containing just the path
   * portion of a string assumed to consist of a path and file name.
   */
  char *ret;
  char *lastchar;

  ret = (char *)malloc( null_strlen(pathfile) + 1 );
  if( ret ){
    // Copy the string over.
    strcpy( ret, pathfile );

    // Locate the last one
    lastchar = strrchr( ret, '/' );
    if( lastchar ){
      // If there was a slash, turn it into a null
      *lastchar = '\0';
    }
  }

  return ret;
}

/***************************************************************************/

char *make_url( char *protocol, char *machine, char *path, char *query ){
  /*
   * Return a newly allocated string containing a correctly formatted URL.
   * "protocol", "machine" and "query" may be NULL, in which they are
   * correctly omitted.
   * Returns NULL if there are problems.
   */

  char *ret;  // Return string
  int  len;   // Length of string

  len = null_strlen(protocol) + null_strlen(machine)
      + null_strlen(path) + null_strlen(query) + 5;

  ret = (char *)malloc( len );
  if( ret ){
    // Allocated ok
    strcpy( ret, "" );

    // Add the protocol string (maybe)
    if( protocol ){
      strcat( ret, protocol );
      if( last_char( protocol ) != ':' ){
        // Mising trailing :, so add
        strcat( ret, ":" );
      }
    }

    // Add the machine (maybe)
    if( machine ){
      strcat( ret, "//" );
      strcat( ret, machine );
      strcat( ret, "/" );
    }

    // Add the path
    strcat( ret, path );

    // Add the query (maybe)
    if( query ){
      strcat( ret, "?" );
      strcat( ret, query );
    }
    
  } else {
    // Out of memory?
  }
  
  return ret;
}

//---------------------------------------------------------------------------

char *make_url( int room_no, char *query ){
  /*
   * A specialized version that returns a newly allocated string that
   * references the room.
   */

  char *ret;       // Return string
  char room[20];   // the file name of the room

  sprintf( room, "%s.html", itoa(room_no) );
  ret = make_url( NULL, NULL, room, query );

  return ret;
}

/***************************************************************************/

int file_exists( char *path ){
  /*
   * Returns 1 if the file exists, and 0 if the file does not
   */
  int ret;  // The return value
  int fd;   // The file descriptor

  fd = open( path, O_CREAT | O_EXCL );
  if( fd < 0 ){
    // File not opened.  Error
    // Assume the file exists
    ret = 1;
  } else {
    // File did not exist, needs to be deleted now
    close( fd );
    unlink( path );
    ret = 0;
  }

  return ret;
}

/***************************************************************************/

void set_permissions(){
  /*
   * Set the umask, file permissions, or whatever
   */
  umask( UMASK );
}

/***************************************************************************/

BacklinkType get_backlink_type( char *s ){
  /*
   * Return a backlink type depending on the string that was sent in.
   * If the string is unparsable, return BL_NONE.
   */
  BacklinkType ret;

  if( strstr( s, "list by title" ) ){
    ret = BL_NAME;
  } else if( strstr( s, "list by number" ) ){
    ret = BL_NUMBER;
  } else if( strstr( s, "don't list" ) ){
    ret = BL_ENTRY;
  } else {
    ret = BL_NONE;
  }

  return ret;
}

//---------------------------------------------------------------------------
  
char *get_backlink_string( BacklinkType t, char *roomword ){
  /*
   * Return a static pointer to a string
   *
   * This string is used in place later, so must be unique.  We will
   * malloc() without deleting, since the program will terminate soon
   * anyway.
   */

  char *ret = (char *)malloc( 160 * sizeof(char) );
    
  switch( t ){
  case BL_NONE:
    sprintf( ret, "Create all new %ss", roomword );
    break;
  case BL_NAME:
    sprintf( ret, "Allow links to previous %s - list by title (Addvanced)",
             roomword );
    break;
  case BL_NUMBER:
    sprintf( ret,
             "Allow links to previous %s - list by number (Addvanced)",
             roomword );
    break;
  case BL_ENTRY:
    sprintf( ret, "Allow links to previous %s - don't list (Addvanced)",
             roomword );
    break;
  }

  return ret;
}

/***************************************************************************/

EvilHtmlType has_evil_html( char *s ){
  /*
   * Indicate what the first kind of HTML formatting that could cause
   * problems is in the string or EVIL_NONE if there is no suspicious
   * formatting.
   */

  typedef struct{
    char* string;
    EvilHtmlType html_type;
  } Match;

  Match match[] = {
    {"<A", EVIL_ANCHOR},
    {"<a", EVIL_ANCHOR},
    {"</A", EVIL_ANCHOR},
    {"</a", EVIL_ANCHOR},
    {"<!", EVIL_COMMENT},
    {"<PRE", EVIL_PRE},
    {"<pre", EVIL_PRE},
    {"<Pre", EVIL_PRE},
    {NULL, EVIL_NONE}
  };

  EvilHtmlType ret = EVIL_NONE;
  int mco = 0;

  while( (match[mco].string) && (ret == EVIL_NONE) ){
    if( strstr( s, match[mco].string ) ){
      ret = match[mco].html_type;
    }
    mco++;
  }

  return ret;
}


/***************************************************************************/

/*
 * $Log: addvutil.C,v $
 * Revision 1.14  1995/05/26  15:55:06  asf
 * Removed default value for make_url() that causes compile errors sometimes.
 *
 * Revision 1.13  1995/02/12  12:40:52  asf
 * Changes to "correctly" support relative addressing.
 *
 * Revision 1.12  1995/01/28  16:59:17  asf
 * Added the has_evil_html() function.
 *
 * Revision 1.11  1995/01/22  12:38:56  asf
 * Changes to make_fname() to allow relative paths.
 *
 * Revision 1.10  1995/01/01  22:36:05  asf
 * Used malloc() in get_backlink_string() instead of a static keyword.
 * This isn't nice... but...
 *
 * Revision 1.9  1995/01/01  22:28:19  asf
 * Increased size of static buffer
 *
 * Revision 1.8  1995/01/01  16:31:30  asf
 * Added support for the "room word"
 *
 * Revision 1.7  1994/09/16  15:41:11  asf
 * Added null_strdup()
 *
 * Revision 1.6  1994/08/16  19:16:57  asf
 * Roms... Rooms... whatever.
 *
 * Revision 1.5  1994/08/16  19:09:52  asf
 * Remember... sizeof() does not return the same value as strlen() in
 * the get_path() function.
 *
 * Revision 1.4  1994/08/16  18:34:31  asf
 * Returns the return value for file_exists()
 *
 * Revision 1.3  1994/08/15  19:32:52  asf
 * Converted all non-recursive calls from strlen() to null_strlen()
 *
 * Revision 1.2  1994/08/08  21:00:08  asf
 * added the backlink functions
 *
 * Revision 1.1  1994/08/04  20:34:03  asf
 * Initial revision
 *
 */
