/*
 * $Id: newproc.C,v 1.14 1995/01/28 16:56:02 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * Process the initial request to deal with a room:
 * 1) Remove the lock  (assume room is under construction if we can't)
 * 2) Make sure the room is not already marked as being editied, and then
 *    mark the room as open for editing
 * 3) Create a .html file that will tell people that wander into the
 *    room that its under construction.  (This is just temporary.)
 * 4) Create the text entry screen
 */

#include <post-query.H>

#include <htmlfile.H>
#include <htmltext.H>

#include "addvutil.H"
#include "setaddv.H"
#include "undo.H"
#include "openfile.H"
#include "lockfile.H"
#include "browser.H"

// Max number of data that can be on the form
#define MAXENTRIES 100

/***************************************************************************/

void hardhat( HtmlFile &f, char *roomword ){
  /*
   * Send to the file text explaining that the room is under construction
   */

  f.title( "Under construction." );
  f.header( 1, "Under construction." );

  f.puts( "Oops... you seem to have wandered into a " );
  f.puts( roomword );
  f.puts( " still being " );
  f.puts( "built.  Try again in a couple of minutes.\n" );

  // @@@ Put in a reload link!
}

/***************************************************************************/

void unknown_error(){
  /*
   * Something strange happened.  No clue what.
   */
  HtmlFile f;

  f.content_type();

  f.title( "Addventure error" );
  f.header( 1, "Addventure error" );

  f.puts( "An error has occurred.  Your reply was not valid.\n" );
}

/***************************************************************************/

void locked_error( char *roomword, char *comment = NULL ){
  /*
   * The file appears to be locked already, so send a message saying
   * its under construction.
   */
  
  HtmlFile f;
  f.content_type();
  
  hardhat( f, roomword );

  if( comment ){
    f.header( 6, comment );
  }
}

/***************************************************************************/

int create_hardhat( int room, char *path, char *roomword ){
  /*
   * Create the file that tells wanderers that this room should be ready
   * soon.
   * Return 1 if success and 0 if error.
   */
  int ret;
  
  FILE *outfile;
  char *filepath;

  // Create the file
  filepath = make_fname( path, room, "html" );
  set_permissions();
  if( outfile = fopen( filepath, "w" ) ){
    // File opened ok
    HtmlFile f( outfile );

    hardhat( f, roomword );
    
    fclose( outfile );
    ret = 1;
  } else {
    // File not opened
    ret = 0;
  }
  
  free( filepath );
  return ret;
}

/***************************************************************************/

void create_textentry( int room, int parent,
                       int backlink, char *roomword,
                       char *filepath, char *urlpath, char *binpath ){
  /*
   * Send a file to the user that contains:
   *   - A title field
   *   - A text entry field
   *   - A field to find out how many options there are
   *   - A way to determine if backlinking is allowed
   */

  char *cgi;     // The executable program
  char *file;    // The executable program with the path of the files
  char *action;  // The URL
  char *undoval;

  // Create the form that is inside the page
  HtmlText form;  // The data inside the form

  form.puts( "You may optionally enter a title:" );
  form.text( "title", 60, 0 );  form.para();

  form.puts( "Describe the scenario:\n" );
  form.textarea( "text", 75, 10 );  form.para();

  form.puts( "How many options will the person have? " );
  form.text( "numoptions" );  form.para();

  if( backlink ){
    // We can backlink

    // There is a list of choices that need to be in the form
    HtmlSNode s1( get_backlink_string( BL_NONE, roomword ), 1 );
    HtmlSNode s2( get_backlink_string( BL_NAME, roomword ) );
    HtmlSNode s3( get_backlink_string( BL_NUMBER, roomword ) );
    HtmlSNode s4( get_backlink_string( BL_ENTRY, roomword ) );

    HtmlSList slist;
    slist.append( s1 );
    slist.append( s2 );
    slist.append( s3 );
    slist.append( s4 );

    form.select( "backlink", &slist );  form.para();

  } else {
    // No backlinking allowed at all
  }
  
  // The "Reject" link
  cgi     = make_fname( binpath, UNDOROOM );
  file    = make_fname( cgi, urlpath );
  Undo undo( room, parent );
  undoval = undo.coded_value();
  action  = make_url( "http", NULL, file, undoval );
  form.link( action, "Cancel" );
  form.puts( " - Do <b>not</b> create this " );
  form.puts( roomword );
  form.puts( ".\n" );
  free( action );
  free( undoval );
  free( file );
  free( cgi );

  // The "Accept" button
  form.submit( "Accept" );
  form.puts( " - Use this description and enter the options." );  form.para();

  // Hidden state values
  form.line();
  form.puts( "Ignore anything below this line.\n" );
  form.hidden( "room", itoa(room) );      form.para();
  form.hidden( "parent", itoa(parent) );  form.para();
  

  HtmlFile f;     // This goes to the standard output

  f.content_type();

  // Create the header and title
  char title[80];
  sprintf( title, "Creating %s %d", roomword, room );
  f.title( title );
  f.header( 1, title );

  // Add the form
  /*
   * The action URL is the name of the processing program, plus
   * the path to the data files.
   */
  cgi    = make_fname( binpath, TEXTPROC );
  file   = make_fname( cgi, urlpath );
  action = make_url( "http", NULL, file );
  f.form( action, METHOD_POST, form.val() );
  free( action );
  free( file );
  free( cgi );
}

/***************************************************************************/

main(){

  PostQuery query( MAXENTRIES );

  int room;
  int parent;
  char *urlpath;   // The path relative to the server root
  char *filepath;  // The absolute path in the file system
  char *binpath;   // Where the cgi scrpts are

  char *browser;   // What browser is in use?

  if( !query.valid() ){
    // Something is wrong
    unknown_error();
  } else {
    // Process the data

    // Get the hidden data
    room   = atoi( query.lookup( "room" ) );
    parent = atoi( query.lookup( "parent" ) );
    
    // Get the data passed by environment variables
    urlpath  = getenv( "PATH_INFO" );
    filepath = getenv( "PATH_TRANSLATED" );
    binpath  = get_path( getenv( "SCRIPT_NAME" ) );
    browser = getenv( "HTTP_USER_AGENT" );

    // Store some browser stats
    BrowserFile binfo( filepath );
    binfo.add( room, browser );

    // Get the game settings
    SetAddv settings( filepath );
    char *roomword = settings.roomword();
    int backlink   = settings.backlink();

    // Find out if we can actually do this
    OpenFile openfile( filepath, room );
    LockFile lockfile( filepath, room );
    if( !lockfile.remove() ){
      // We couldn't delete it - room was locked already
      locked_error( roomword, "Lockfile deleted" );
    } else if( openfile.exists() ){
      // Room is already being created
      locked_error( roomword, "Working file exists" );
    } else {
      // We're fine.  Create the file, openfile, and output
      create_hardhat( room, filepath, roomword );
      openfile.create( parent );
      create_textentry( room, parent, backlink, roomword, 
                        filepath, urlpath, binpath );
    }
  }
  
}

/***************************************************************************/

/*
 * $Log: newproc.C,v $
 * Revision 1.14  1995/01/28  16:56:02  asf
 * Added logging information about browsers used.
 *
 * Expanded the text entry field of the title.
 *
 * Revision 1.13  1995/01/01  16:37:32  asf
 * Added support for the "room word"
 *
 * Revision 1.12  1994/11/06  14:34:54  asf
 * Added parameter to the locked error
 *
 * Revision 1.11  1994/10/01  15:45:34  asf
 * Switched to use of LockFile and OpenFile.
 *
 * Revision 1.10  1994/09/27  17:08:51  asf
 * Added parent parameter to the openfile_create() function
 *
 * Revision 1.9  1994/09/16  15:10:25  asf
 * Do not allow the selection of how to backlink if backlinking is
 * not allowed.
 *
 * Revision 1.8  1994/08/25  01:38:26  asf
 * Standardized accept/reject lines.
 *
 * Revision 1.7  1994/08/25  01:09:23  asf
 * Added openfile support.
 *
 * Revision 1.6  1994/08/20  22:04:59  asf
 * Added reject link
 *
 * Revision 1.5  1994/08/14  14:33:30  asf
 * Added CGI-script name definition
 *
 * Revision 1.4  1994/08/08  20:54:52  asf
 * Added the use of the path to the CGI scripts.
 * Changed some hardcoded strings to call a utility function.
 *
 * Revision 1.3  1994/08/06  09:44:18  asf
 * Added para after the textarea.
 * Textarea should be 75x10, not 10x75
 *
 * Revision 1.2  1994/08/06  09:36:30  asf
 * Ensured that the content type header is sent.
 *
 * Revision 1.1  1994/08/06  08:44:02  asf
 * Initial revision
 *
 */
