/*
 * $Id: setaddv.C,v 1.10 1995/01/28 17:00:10 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1991-94, 1995  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include "version.H"
#include "addvutil.H"
#include "setaddv.H"

#define LABEL_VERSION      "version"
#define LABEL_STARTED      "DateStarted"

#define LABEL_URLPATH      "URLPath"
#define LABEL_FILEPATH     "FilePath"
#define LABEL_BINPATH      "BinPath"

#define LABEL_BACKLINK     "Backlink"
#define LABEL_LINKDEFAULT  "LinkDefault"
#define LABEL_CLEANUP      "Cleanup"
#define LABEL_GOBACK       "Goback"
#define LABEL_TITLE        "Title"
#define LABEL_EMAIL        "Contact"
#define LABEL_HTML         "EvilHTML"
#define LABEL_ROOMWORD     "RoomW"

/***************************************************************************/

SetAddv::SetAddv( char *path ):Setting( path ){
}
     
//---------------------------------------------------------------------------
  
SetAddv::~SetAddv(){
}
  
//---------------------------------------------------------------------------

void SetAddv::set( Opt &command_line ){
  /*
   * Read the values from the command line and create a new file
   */

  // Always set the version and other standard info
  setLabel( LABEL_VERSION, VERSION_NUM );
  
  /*
   * The following command line values are parsed by the newgame module,
   * but are stored in the file here.  We know they must exist.
   *
   * -filepath
   * -urlpath
   * -binpath
   */

  char *filepath = command_line.get_value( "-filepath" );
  char *urlpath  = command_line.get_value( "-urlpath" );
  char *binpath  = command_line.get_value( "-binpath" );
  setLabel( LABEL_FILEPATH, filepath );
  setLabel( LABEL_URLPATH,  urlpath );
  setLabel( LABEL_BINPATH,  binpath );

  // Can we backlink?
  // -backlink or -nobacklink
  setboolean( command_line, "backlink", LABEL_BACKLINK, 1 );

  
  // Should backlinking be the default?
  // -linkdefault -nolinkdefault
  setboolean( command_line, "linkdefault", LABEL_LINKDEFAULT, 0 );


  // Should anchors be allowed in options?
  // -anchoroption -noanchoroption
  setboolean( command_line, "evilhtml", LABEL_HTML, 0 );
  
  // When to clean up?
  // (We convert atoi and then itoa to make sure we have a number)
  printf( "%s: ", LABEL_CLEANUP );
  int cleanup = 24 * 60 * 60;  //Number of seconds in a day (default)
  char *cleanup_string = command_line.get_value( "-cleanup" );
  if( cleanup_string ){
    // A value was entered
    printf( "Set to: " );
    cleanup = atoi( cleanup_string );
  } else {
    // No value entered
    printf( "Using default: " );
  }
  printf( "%d seconds. ", cleanup );
  printf( "(Don't forget to run the cleanup program.)\n" );
  setLabel( LABEL_CLEANUP, itoa( cleanup ) );


  // Should we be able to go back to the previous room?
  // -goback       - always allow
  // -nogoback     - never allow
  // -singlegoback - go back only if we are guaranteed there is only
  //                 one parent.  This happens if we have backlinking
  //                 off (in which case we change this to -goback), or
  //                 if the room may be linked to (which is determined
  //                 at runtime).
  printf( "%s: ", LABEL_GOBACK );
  int goback_value   = -1;  // 1, 0, -1  (go, nogo, single)
  char *goback       = command_line.get_value( "-goback" );
  char *nogoback     = command_line.get_value( "-nogoback" );
  char *singlegoback = command_line.get_value( "-singlegoback" );
  
  // Determine if we use defaults or what the value to use is.
  if((goback&&nogoback) || (goback&&singlegoback) || (nogoback&&singlegoback)){
    // Can only have one of these
    printf( "Can only specify one of -goback, -nogoback, or -singlegoback. " );
    printf( "Using default: " );
  } else if( (!goback) && (!nogoback) && (!singlegoback) ){
    // Use the default
    printf( "Using default: " );
  } else {
    // Set the value
    printf( "Set to: " );
    if( goback ){
      goback_value = 1;
    } else if( nogoback ){
      goback_value = 0;
    } else if( singlegoback ){
      goback_value = -1;
    }
  }
  
  // Set it in the file and print the value
  switch( goback_value ){
  case 1:
    printf( "On\n" );
    setLabel( LABEL_GOBACK, "on" );
    break;
  case 0:
    printf( "Off\n" );
    setLabel( LABEL_GOBACK, "off" );
    break;
  case -1:
    printf( "Single parent only\n" );
    setLabel( LABEL_GOBACK, "single" );
    break;
  }


  // Set the (optional) game title
  printf( "%s: ", LABEL_TITLE );
  char *title = command_line.get_value( "-title" );
  if( null_strlen( title ) ){
    printf( "%s\n", title );
    setLabel( LABEL_TITLE, title );
  } else {
    // No title was given
    printf( "(None)\n" );
  }

  // Set what word we will use for "room"
  printf( "%s: ", LABEL_ROOMWORD );
  char *roomw = command_line.get_value( "-roomw" );
  if( null_strlen( roomw ) ){
    printf( "%s\n", roomw );
    setLabel( LABEL_ROOMWORD, roomw );
  } else {
    // Use the default
    printf( "room\n" );
    setLabel( LABEL_ROOMWORD, "room" );
  }
  
}

//---------------------------------------------------------------------------

char *SetAddv::version(){
  /*
   * What version was this game created with?
   */
  char *ret = getLabel( LABEL_VERSION, "unknown" );
  return ret;
}

//---------------------------------------------------------------------------

char *SetAddv::date_started(){
  /*
   * When was the game started?
   */
  char *ret = getLabel( LABEL_STARTED, NULL );
  return ret;
}

//---------------------------------------------------------------------------

int SetAddv::backlink(){
  /*
   * Should we backlink at all?
   */
  int ret = getboolean( LABEL_BACKLINK, 1 );
  return ret;
}

//---------------------------------------------------------------------------

int SetAddv::linkdefault(){
  /*
   * Should we allow linking to a room by default?
   */
  int ret = getboolean( LABEL_LINKDEFAULT, 0 );
  return ret;
}

//---------------------------------------------------------------------------

int SetAddv::evil_html(){
  /*
   * Should we allow various HTML?
   */
  int ret = getboolean( LABEL_HTML, 0 );
  return ret;
}

//---------------------------------------------------------------------------

int SetAddv::cleanup_after(){
  /*
   * When should we restore a stale room?
   */
  int ret;

  char *value;

  value = getLabel( LABEL_CLEANUP );
  if( value ){
    ret = atoi( value );
  } else {
    ret = 24 * 60 * 60;    // One day
  }

  return ret;
}

//---------------------------------------------------------------------------

int SetAddv::goback(){
  /*
   * When should we provide a link to the parent?
   */
  int ret;
  int def = -1;

  char *value;

  value = getLabel( LABEL_GOBACK );
  if( value ){
    if( !strcmp( value, "on" ) ){
      ret = 1;
    } else if( !strcmp( value, "off" ) ){
      ret = 0;
    } else if( !strcmp( value, "single" ) ){
      ret = -1;
    } else {
      ret = def;
    }
  } else {
    ret = def;
  }

  return ret;
}

//---------------------------------------------------------------------------

char *SetAddv::game_title(){
  /*
   * Game title  (NULL if none)
   */
  char *ret = getLabel( LABEL_TITLE, NULL );
  return ret;
}

//---------------------------------------------------------------------------

char *SetAddv::contact(){
  /*
   * Who is in charge of this game?
   */
  char *ret = getLabel( LABEL_EMAIL, NULL );
  return ret;
}

//---------------------------------------------------------------------------

char *SetAddv::roomword(){
  /*
   * What word do we use for "room"
   */
  char *ret = getLabel( LABEL_ROOMWORD, "room" );
  return ret;
}

//---------------------------------------------------------------------------

int  SetAddv::getboolean( char *label, int def ){
  int ret;
  char *value;

  value = getLabel( label );
  if( value ){
    if( !strcmp( value, "on" ) ){
      ret = 1;
    } else if (!strcmp( value, "off" ) ){
      ret = 0;
    } else {
      ret = def;
    }
  } else {
    ret = def;
  }

  return ret;
}

//---------------------------------------------------------------------------

void SetAddv::setboolean( Opt &command_line, char *cl_value,
                          char *label, int def ){

  // The real command line argumens to look for have a hyphen, and there
  // is a negative value
  char *positive_value = (char *)malloc( strlen(cl_value) + 2 );
  sprintf( positive_value, "-%s", cl_value );
  char *negative_value = (char *)malloc( strlen(cl_value) + 4 );
  sprintf( negative_value, "-no%s", cl_value );
    
  printf( "%s: ", label );
  char *positive = command_line.get_value( positive_value );
  char *negative = command_line.get_value( negative_value );
  int  set_value = def;
  if( positive && negative ){
    // Conflicing
    printf( "Can't specify %s and %s.  Using default: ",
            positive_value, negative_value );
  } else if( (!positive) && (!negative) ){
    // No response
    printf( "Using default: " );
  } else {
    printf( "Set to: " );
    if( positive ) set_value = 1;
    if( negative ) set_value = 0;
  }
  if( set_value ){
    printf( "On\n" );
    setLabel( label, "on" );
  } else {
    printf( "Off\n" );
    setLabel( label, "off" );
  }

  // Free the memory
  free( negative_value );
  free( positive_value );
}


/***************************************************************************/

/*
 * $Log: setaddv.C,v $
 * Revision 1.10  1995/01/28  17:00:10  asf
 * Changed the "anchor option" to "evil html".
 *
 * Revision 1.9  1995/01/01  16:30:52  asf
 * Added support for the "room word"
 *
 * Revision 1.8  1994/12/17  21:52:21  asf
 * The paths are now stored as well.
 *
 * Revision 1.7  1994/12/17  21:37:01  asf
 * Fixed a typo in cleanup.
 *
 * Revision 1.6  1994/12/17  20:11:16  asf
 * Added the ability to put or deny anchors in optiosn.
 *
 * Revision 1.5  1994/10/03  13:56:38  asf
 * Replaced hardcoded version number with the definition from version.H
 *
 * Revision 1.4  1994/10/01  19:36:07  asf
 * Added version(), date_started(), and contact() accessors.
 *
 * Revision 1.3  1994/09/16  14:16:50  asf
 * Added the accessor functions.
 *
 * Revision 1.2  1994/09/15  11:24:02  asf
 * Added processing for [no]backlink, [no]linkdefault, cleanup, and
 * [no|single]goback.
 * Changed parameter name from "default" to "def"
 *
 * Revision 1.1  1994/09/14  13:49:04  asf
 * Initial revision
 *
 */
