/*
 * $Id: textproc.C,v 1.22 1995/05/26 15:58:40 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * Process the text entry screen:
 * 1) Ensure that text was entered and the number of options is greater
 *    than zero
 * 2) Ensure that the temporary file does not already exist and that the
 *    room was correctly opened.
 * 3) Create the first part of the room in a temporary file
 * 4) Create the option entry screen
 *    - May need to include a backlink option
 */

#include <htmlfile.H>
#include <htmltext.H>

#include <post-query.H>

#include "setaddv.H"
#include "addvutil.H"
#include "backlink.H"
#include "undo.H"
#include "openfile.H"
#include "error.H"

/***************************************************************************/

int temp_exists( int room, char *path ){
  /*
   * Return 1 if the temporary file for the room already exists, 
   * and 0 if not.
   */
  int ret;
  char *tempfile;

  tempfile = make_fname( path, room, "temp" );
  ret = file_exists( tempfile );
  free( tempfile );

  return ret;
}

/***************************************************************************/

int create_temp( char *path, int room, char *roomword,
                 char *titleroom, char *gametitle, char *title,
                 char *text ){
  /*
   * Create the temp file that will become the room
   * Return 1 if no problems, 0 if problems
   */
  int ret;
  
  char *filepath;     // Where to save it
  FILE *file;

  char roomstr[100];   // A string containing the game title and room number

  filepath = make_fname( path, room, "temp" );
  set_permissions();
  if( file = fopen( filepath, "w" ) ){
    // File opened
    HtmlFile f( file );

    // Print title and header text
    f.title( titleroom );
    if( title ){
      f.header( 1, title );
    }
    if( gametitle ){
      sprintf( roomstr, "%s - %s %d", gametitle, roomword, room );
    } else {
      sprintf( roomstr, "%s %d", roomword, room );
    }
    f.header( 2, roomstr );

    // Print the text
    f.raw( text );  f.para();
    
    fclose( file );
    ret = 1;
  } else {
    // File not opened
    ret = 0;
  }
                   
  free( filepath );
}

/***************************************************************************/

HtmlText *create_optionline( int opt_no, BacklinkType link, BackLink *rooms,
                             char *roomword ){
  /*
   * Return an object that contains the contents
   * of the part of a form prompting for an option and, possibly,
   * a backlink.
   */
  HtmlText *ret;

  char optionname[20];  // The name to be used for the option entry area
  char linkname[20];    // The name to be used for the link area

  // Each data entry field needs a unique name that is based on the
  // option number
  sprintf( optionname, "option%d", opt_no );
  sprintf( linkname,   "link%d",   opt_no );

  ret = new HtmlText;

  ret->text( optionname, 60, 0 );

  switch( link ){
  case BL_NONE:
    // No backlinking
    break;
  case BL_NAME:
    // Show the list by name  (assume already sorted)
    ret->select( linkname, rooms->roomlist(), 5 );
    break;
  case BL_NUMBER:
    // Show the list by room number  (assume already sorted)
    ret->select( linkname, rooms->roomlist(), 5 );
    break;
  case BL_ENTRY:
    // Give only a text entry area
    ret->puts( roomword );
    ret->puts( " number: " );
    ret->text( linkname );
    ret->puts( "(Optional)" );
    break;
  }
  
  return ret;
}

//---------------------------------------------------------------------------

void create_optentry( int room, int parent, int numopt, char *backlink,
                      char *urlpath, char *filepath, char *binpath,
                      char *titleroom ){
  /*
   * Create the option entry room.
   *
   * If the backlink type says we need a list, load the list and sort
   * it as appropriate.
   *
   * Create a numbered list containing each option entry area.  The
   * option entry areas are generated seperately.
   *
   * Create the rest of the form.
   */
  int co;

  char *cgi;     // The executable program
  char *file;    // The executable program with the path of the files
  char *action;  // The URL
  char *undoval;

  SetAddv s( filepath );    // The file that contains settings and defaults
  char *roomword = s.roomword();

  BackLink *rooms;
  BacklinkType link;

  // Load the backlink list if necessary (and allowed)
  if( s.backlink() ){
    // Backlinking is allowed
    link = get_backlink_type( backlink );
    if( (link == BL_NAME ) || (link == BL_NUMBER) ){

      // Load the list
      rooms = new BackLink( filepath, roomword );

      // Sort it
      if( link == BL_NAME ){
        rooms->sort_name();
      } else if( link == BL_NUMBER ){
        rooms->sort_number();
      }

    } else {
      // We don't need to load the list
      rooms = NULL;
    }
  } else {
    // No backlinking allowed
    link = BL_NONE;
    rooms = NULL;
  }
  
  // Create the list that will become the bulleted option entry areas
  HtmlList options;
  for( co=1; co<=numopt; co++ ){
    // Create the form data for an option
    HtmlText *option_text = create_optionline( co, link, rooms, roomword );

    // Create the node and add it
    HtmlNode *node = new HtmlNode( option_text->val() );
    options.append( *node );
  }

  // Create the form
  HtmlText form;

  // The options
  form.puts( "Options:\n" );
  form.numlist( &options );

  // Signature
  form.puts( "You may sign the " );
  form.puts( roomword );
  form.puts( " if you wish: " );
  form.text( "signature", 40, 0 );  form.para();

  // Does the author want email when the room is extended?
  // Set the checkbox
  form.checkbox( "sendmail", 0, "1" );
  form.puts( "Send mail when the path is extended.  (Addvanced option) " );
  form.puts( "You must put your email address here to do this:" );
  form.text( "email", 40, 0 );   form.para();
  
  // Can we link to this room?
  // Make sure we can backlink at all!
  if( s.backlink() ){
    // Backlinking allowed

    // Find out from the option file
    int link_default = s.linkdefault();
    
    // Set the checkbox
    form.checkbox( "linkhere", link_default, "1" );
    form.puts( "Link to this " );
    form.puts( roomword );
    form.puts( " later.  (Addvanced option).\n" );
  } else {
    // No backlinking allowed
  }
  
  // The "Reject" link
  cgi     = make_fname( binpath, UNDOROOM );
  file    = make_fname( cgi, urlpath );
  Undo undo( room, parent );
  undoval = undo.coded_value();
  action  = make_url( "http", NULL, file, undoval );
  form.link( action, "Cancel" );
  form.puts( " - Do <b>not</b> create this " );
  form.puts( roomword );
  form.puts( ".\n" );
  free( action );
  free( undoval );
  free( file );
  free( cgi );

  // The "Accept" button
  form.submit( "Accept" );
  form.puts( " - Use these options and finish creating the " );
  form.puts( roomword );
  form.puts( "." );
  form.para();

  // Hidden state values
  form.line();
  form.puts( "Ignore anything below this line.\n" );
  form.hidden( "room", itoa(room) );          form.para();
  form.hidden( "parent", itoa(parent) );      form.para();
  form.hidden( "numoptions", itoa(numopt) );  form.para();
  form.hidden( "title", titleroom );          form.para();
  form.hidden( "backlink", backlink );        form.para();
  
  // Create a title
  char title[80];
  sprintf( title, "Enter options for %s %d", roomword, room );

  // Create the action
  cgi    = make_fname( binpath, OPTPROC );
  file   = make_fname( cgi, urlpath );
  action = make_url( "http", NULL, file );

  // Create and send the rest of the document
  HtmlFile f;   // Send to standard output

  f.content_type();

  f.title( title );
  f.header( 1, title );
  f.form( action, METHOD_POST, form.val() );

  free( action );
  free( file );
  free( cgi );
}

/***************************************************************************/

#define MAXENTRIES 100

main(){
  PostQuery query( MAXENTRIES );

  int room;
  int parent;
  char *urlpath;   // The path relative to the server root
  char *filepath;  // The absolute path in the file system
  char *binpath;   // Where the cgi scrpts are

  char *gametitle;      // The name of the entire game
  char *title;          // The title the user entered
  char *titleroom;      // The title with room number in brackets
  char *gametitleroom;  // The game title, title, and room
  
  char *text;           // Text entered
  int  numopt;          // How many options
  char *backlink;       // What kind of backlinking to do, if any

  if( !query.valid() ){
    // Something is wrong with the query
    addventure_error( TEXTPROC, ADDVERR_UNKNOWN );
  } else {
    // Process the data

    // Get the hidden data
    room   = atoi( query.lookup( "room" ) );
    parent = atoi( query.lookup( "parent" ) );

    // Get the data passed by environment variables
    urlpath  = getenv( "PATH_INFO" );
    filepath = getenv( "PATH_TRANSLATED" );
    binpath  = get_path( getenv( "SCRIPT_NAME" ) );

    OpenFile openfile( filepath, room );

    SetAddv setting( filepath );
    char *roomword = setting.roomword();
    int allow_html = setting.evil_html();
    
    if( !openfile.exists() ){
      // The room wasn't started correctly
      addventure_error( TEXTPROC, ADDVERR_NOT_EXIST, roomword );
    } else {
      // Get the other data from the form
      title    = query.lookup( "title" );
      text     = query.lookup( "text" );
      numopt   = atoi( query.lookup( "numoptions" ) );
      backlink = query.lookup( "backlink" );

      // Add the brackets after the title
      if( title ){
        // The user entered a title
        titleroom = (char *)malloc( null_strlen(title) + 20 );
        sprintf( titleroom, "%s [%s %d]", title, roomword, room );
      } else {
        // No title entered
        titleroom = (char *)malloc( 20 );
        sprintf( titleroom, "[%s %d]", roomword, room );
      }

      // Get the title of the game and create the full title
      SetAddv setting( filepath );
      gametitle = setting.game_title();
      if( gametitle ){
        gametitleroom = (char *)malloc(strlen(gametitle)+strlen(titleroom)+5 );
        sprintf( gametitleroom, "%s: %s", gametitle, titleroom );
      } else {
        // No title to the game
        gametitleroom = null_strdup( titleroom );
      }

      if( !(text && (numopt > 0 )) ){
        // Either no text or no options
        addventure_error( TEXTPROC, ADDVERR_EMPTY, roomword );
      } else if( temp_exists( room, filepath ) ){
        // The temp file already exists
        addventure_error( TEXTPROC, ADDVERR_EXISTS, roomword );
      } else if( (!allow_html) && (has_evil_html( text )) ){
        // There is HTML that we don't like
        addventure_error( TEXTPROC, ADDVERR_HTML, roomword );
      } else {
        // Create the file and screen for the user
        create_temp( filepath, room, roomword,
                     gametitleroom, gametitle, title, text );
        create_optentry( room, parent, numopt, backlink,
                        urlpath, filepath, binpath,
                        titleroom );
      }

      free( titleroom );
      free( gametitleroom );
    }
  }
}

/***************************************************************************/

/*
 * $Log: textproc.C,v $
 * Revision 1.22  1995/05/26  15:58:40  asf
 * Fixed some potential compile problems in create_optionline().
 *
 * Revision 1.21  1995/01/28  16:58:44  asf
 * Added support to check for evil HTML.
 *
 * Extended some of the text entry fields.
 *
 * Revision 1.20  1995/01/01  22:52:59  asf
 * More converting to "room word" that was missed in create_optionline()
 *
 * Revision 1.19  1995/01/01  17:37:17  asf
 * Added the "room word" to the error messages
 *
 * Revision 1.18  1995/01/01  16:34:37  asf
 * Added support for the "room word"
 *
 * Revision 1.17  1994/12/15  20:59:19  asf
 * Added a new box to enter an email address, seperate from the signature.
 *
 * Revision 1.16  1994/11/06  13:09:54  asf
 * Added "sendmail" checkbox.
 *
 * Revision 1.15  1994/10/01  15:52:54  asf
 * Switched to use of LockFile and OpenFile.
 *
 * Revision 1.14  1994/09/16  15:44:22  asf
 * The game title is now part of the title and header.
 *
 * Revision 1.13  1994/09/16  15:09:28  asf
 * Several changes that configure things correctly if backlinking is or
 * is not allowed.
 * The method to determine if linking to a room is on by default is now done
 * using the SetAddv object instead of reading the game defaults file
 * directly.
 *
 * Revision 1.12  1994/08/26  09:07:54  asf
 * Converted to error package.
 *
 * Revision 1.11  1994/08/25  01:38:45  asf
 * Standardizes accept/reject lines.
 *
 * Revision 1.10  1994/08/25  01:09:35  asf
 * Added openfile support.
 *
 * Revision 1.9  1994/08/23  20:47:29  asf
 * Changed two structure references to pointer-to-structure references.
 *
 * Revision 1.8  1994/08/20  22:03:07  asf
 * Added reject link
 *
 * Revision 1.7  1994/08/15  19:44:27  asf
 * Converted from strlen() to null_strlen()
 *
 * Revision 1.6  1994/08/14  14:33:14  asf
 * Added CGI-script name definition
 *
 * Revision 1.5  1994/08/13  19:16:06  asf
 * Calls optproc, as it should, instead of textproc.
 *
 * Revision 1.4  1994/08/13  19:11:42  asf
 * The BackLink object now knows the name of the file, it just needs the path.
 *
 * Revision 1.3  1994/08/09  18:10:43  asf
 * Put in the hidden state of the backlink string.
 *
 * Revision 1.2  1994/08/09  17:02:56  asf
 * Read the LinkDefault setting from the options file
 *
 * Revision 1.1  1994/08/08  20:49:45  asf
 * Initial revision
 *
 */
